"""
Django settings for subject project.

Generated by 'django-admin startproject' using Django 2.2.3.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""
import os
from sqlalchemy.types import CHAR

from iam.ext.django.defaults import *

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.getenv('DEBUG') == '1'


# Application definition

INSTALLED_APPS = [
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    'aorta.ext.django',
    'iam.ext.django',
    'iam.ext.app',
    'aldjemy',
    'social_django'
]

MIDDLEWARE = [
    'csp.middleware.CSPMiddleware',
    'unimatrix.ext.django.lib.middleware.HealthCheckMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'iam.lib.middleware.IAMMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sites.middleware.CurrentSiteMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'iam.ext.app.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'environment': 'iam.ext.django.lib.template.environment',
            'context_processors': [
                'django.contrib.messages.context_processors.messages',
                'iam.lib.template.policies',
                'social_django.context_processors.backends',
            ]
        },
    },
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'unimatrix.ext.django.wsgi.application'


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', 'English'),
    ('nl', 'Dutch'),
)

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = 'static'

ALDJEMY_DATA_TYPES = {
    'UUIDField': CHAR,
}

IAM_DEBUG_PATH = ''

IAM_REALM_SETTINGS = {
    'default': {
        'public': True,
        'sso': {
            'providers': [
                'google-oauth2',
            ]
        }
    },
    'backoffice': {
        'domains': [
            'wizardsofindustry.com'
        ],
        'public': False,
        'sso': {
            'providers': [
                'google-oauth2',
            ]
        }
    }
}

CACHE_HOST = os.getenv('CACHE_HOST')

CACHE_PORT = os.getenv('CACHE_PORT')

REDIS_DATABASE = os.getenv('REDIS_DATABASE')

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": f"redis://{CACHE_HOST}:{CACHE_PORT}/{REDIS_DATABASE}",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    }
}


CSP_DEFAULT_SRC = (
    "'self'",
)

CSP_SCRIPT_SRC = (
    "'self'",
    'code.jquery.com',
    'kit.fontawesome.com',
    'stackpath.bootstrapcdn.com',
)

CSP_FONT_SRC = (
    "'self'",
    'kit-free.fontawesome.com',
    'maxcdn.bootstrapcdn.com',
)

CSP_STYLE_SRC = (
    "'self'",
    'kit-free.fontawesome.com',
    'kit.fontawesome.com',
    'maxcdn.bootstrapcdn.com',
    'stackpath.bootstrapcdn.com',
)

CSP_IMG_SRC = (
    "'self'",
)



"""Declares :class:`ReadonlyModelAdminMixin`."""


class ReadonlyModelAdminMixin:

    def get_actions(self, request):
        actions = super().get_actions(request)
        del_action = "delete_selected"
        if del_action in actions:
            del actions[del_action]
        return actions

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        pass

    def delete_model(self, request, obj):
        pass

    def save_related(self, request, form, formsets, change):
        pass

    def get_readonly_fields(self, request, obj=None):
        return list(set(
            [field.name for field in self.opts.local_fields] +
            [field.name for field in self.opts.local_many_to_many]
        ))



"""Declares :class:`TemplateService`."""
from unimatrix.ext.django import services


class TemplateService(services.TemplateService):
    """Provides an interface to render templates."""
    default_prefix = 'iam'



#
# Copyright (C) 2019-2020 Cochise Ruhulessin
#
# This file is part of python-iam.
#
# python-iam is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# python-iam is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-iam.  If not, see <https://www.gnu.org/licenses/>.
"""Declares :class:`Principal`."""
import re

from .resource import Resource


class Principal(Resource):
    """Identifies a Subject."""
    api_version = 'iam.unimatrixone.io/v1'
    pattern = '^$'

    @classmethod
    def iscapable(cls, value):
        """Return if the value is correct for the principal type."""
        return bool(re.match(cls.pattern, value))

    def __init__(self):
        self.subject = None

    def isresolved(self):
        """Return a boolean indicating if the :term:`Principal` is resolved
        to a :term:`Subject`.
        """
        return self.subject is not None

    def resolve(self, realm, resolver):
        """Resolves the :term:`Principal` to a :term:`Subject` using
        the given resolver.
        """
        self.subject = resolver.resolve(realm, self)
        return self.subject

    def __repr__(self):
        return f"<iam.{self.__class__.__name__}: {self.value}>"



"""Declares :class:`~django.contrib.admin.ModelAdmin` implementations."""
from .acceptedplatformagreement import AcceptedPlatformAgreementAdmin
from .platformagreementlanguage import PlatformAgreementLanguageAdmin
from .subjectregistration import SubjectRegistrationAdmin
